@@ -22,6 +22,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
+#include <float.h>
+#include <math.h>
 
 
 #define TRACKER_TYPE_SPARQL_BUILDER (tracker_sparql_builder_get_type ())
@@ -101,6 +103,7 @@ void tracker_sparql_builder_object_string (TrackerSparqlBuilder* self, const cha
 void tracker_sparql_builder_object_boolean (TrackerSparqlBuilder* self, gboolean literal);
 void tracker_sparql_builder_object_int64 (TrackerSparqlBuilder* self, gint64 literal);
 void tracker_sparql_builder_object_date (TrackerSparqlBuilder* self, time_t* literal);
+void tracker_sparql_builder_object_double (TrackerSparqlBuilder* self, double literal);
 static void _vala_array_add7 (TrackerSparqlBuilderState** array, int* length, int* size, TrackerSparqlBuilderState value);
 void tracker_sparql_builder_object_blank_open (TrackerSparqlBuilder* self);
 static void _vala_array_add8 (TrackerSparqlBuilderState** array, int* length, int* size, TrackerSparqlBuilderState value);
@@ -396,6 +399,24 @@ void tracker_sparql_builder_object_date (TrackerSparqlBuilder* self, time_t* lit
 }
 
 
+static char* double_to_string (double self) {
+	char* result;
+	gint _tmp0__length1;
+	gchar* _tmp0_;
+	char* _tmp1_;
+	result = (_tmp1_ = g_strdup (g_ascii_dtostr ((_tmp0_ = g_new0 (gchar, G_ASCII_DTOSTR_BUF_SIZE), _tmp0__length1 = G_ASCII_DTOSTR_BUF_SIZE, _tmp0_), G_ASCII_DTOSTR_BUF_SIZE, self)), _tmp0_ = (g_free (_tmp0_), NULL), _tmp1_);
+	return result;
+}
+
+
+void tracker_sparql_builder_object_double (TrackerSparqlBuilder* self, double literal) {
+	char* _tmp0_;
+	g_return_if_fail (self != NULL);
+	tracker_sparql_builder_object (self, _tmp0_ = double_to_string (literal));
+	_g_free0 (_tmp0_);
+}
+
+
 static void _vala_array_add7 (TrackerSparqlBuilderState** array, int* length, int* size, TrackerSparqlBuilderState value) {
 	if ((*length) == (*size)) {
 		*size = (*size) ? (2 * (*size)) : 4;
