@@ -1635,45 +1635,10 @@ gboolean tracker_turtle_reader_next (TrackerTurtleReader* self, GError** error)
 
 void tracker_turtle_reader_load (const char* path, GError** error) {
 	GError * _inner_error_;
+	TrackerTurtleReader* reader;
 	g_return_if_fail (path != NULL);
 	_inner_error_ = NULL;
-	{
-		TrackerTurtleReader* reader;
-		tracker_data_begin_transaction ();
-		reader = tracker_turtle_reader_new (path, &_inner_error_);
-		if (_inner_error_ != NULL) {
-			goto __finally3;
-		}
-		while (TRUE) {
-			gboolean _tmp0_;
-			_tmp0_ = tracker_turtle_reader_next (reader, &_inner_error_);
-			if (_inner_error_ != NULL) {
-				_g_object_unref0 (reader);
-				goto __finally3;
-			}
-			if (!_tmp0_) {
-				break;
-			}
-			if (reader->priv->_object_is_uri) {
-				tracker_data_insert_statement_with_uri (reader->priv->_subject, reader->priv->_predicate, reader->priv->_object, &_inner_error_);
-				if (_inner_error_ != NULL) {
-					_g_object_unref0 (reader);
-					goto __finally3;
-				}
-			} else {
-				tracker_data_insert_statement_with_string (reader->priv->_subject, reader->priv->_predicate, reader->priv->_object, &_inner_error_);
-				if (_inner_error_ != NULL) {
-					_g_object_unref0 (reader);
-					goto __finally3;
-				}
-			}
-		}
-		_g_object_unref0 (reader);
-	}
-	__finally3:
-	{
-		tracker_data_commit_transaction ();
-	}
+	reader = tracker_turtle_reader_new (path, &_inner_error_);
 	if (_inner_error_ != NULL) {
 		if (((_inner_error_->domain == G_FILE_ERROR) || (_inner_error_->domain == TRACKER_SPARQL_ERROR)) || (_inner_error_->domain == TRACKER_DATA_ERROR)) {
 			g_propagate_error (error, _inner_error_);
@@ -1684,6 +1649,55 @@ void tracker_turtle_reader_load (const char* path, GError** error) {
 			return;
 		}
 	}
+	while (TRUE) {
+		gboolean _tmp0_;
+		_tmp0_ = tracker_turtle_reader_next (reader, &_inner_error_);
+		if (_inner_error_ != NULL) {
+			if (((_inner_error_->domain == G_FILE_ERROR) || (_inner_error_->domain == TRACKER_SPARQL_ERROR)) || (_inner_error_->domain == TRACKER_DATA_ERROR)) {
+				g_propagate_error (error, _inner_error_);
+				_g_object_unref0 (reader);
+				return;
+			} else {
+				_g_object_unref0 (reader);
+				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
+				g_clear_error (&_inner_error_);
+				return;
+			}
+		}
+		if (!_tmp0_) {
+			break;
+		}
+		if (reader->priv->_object_is_uri) {
+			tracker_data_insert_statement_with_uri (reader->priv->_subject, reader->priv->_predicate, reader->priv->_object, &_inner_error_);
+			if (_inner_error_ != NULL) {
+				if (((_inner_error_->domain == G_FILE_ERROR) || (_inner_error_->domain == TRACKER_SPARQL_ERROR)) || (_inner_error_->domain == TRACKER_DATA_ERROR)) {
+					g_propagate_error (error, _inner_error_);
+					_g_object_unref0 (reader);
+					return;
+				} else {
+					_g_object_unref0 (reader);
+					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
+					g_clear_error (&_inner_error_);
+					return;
+				}
+			}
+		} else {
+			tracker_data_insert_statement_with_string (reader->priv->_subject, reader->priv->_predicate, reader->priv->_object, &_inner_error_);
+			if (_inner_error_ != NULL) {
+				if (((_inner_error_->domain == G_FILE_ERROR) || (_inner_error_->domain == TRACKER_SPARQL_ERROR)) || (_inner_error_->domain == TRACKER_DATA_ERROR)) {
+					g_propagate_error (error, _inner_error_);
+					_g_object_unref0 (reader);
+					return;
+				} else {
+					_g_object_unref0 (reader);
+					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
+					g_clear_error (&_inner_error_);
+					return;
+				}
+			}
+		}
+	}
+	_g_object_unref0 (reader);
 }
 
 
