@@ -49,8 +49,6 @@ typedef struct ProcessFileData ProcessFileData;
 struct ProcessFileData {
 	TrackerMinerFiles *miner;
 	TrackerSparqlBuilder *sparql;
-	TrackerMinerFSDoneCb callback;
-	gpointer callback_data;
 	GCancellable *cancellable;
 	GFile *file;
 	DBusGProxyCall *call;
@@ -105,17 +103,11 @@ static void     mount_point_removed_cb        (TrackerStorage       *storage,
 					       const gchar          *udi,
 					       const gchar          *mount_point,
 					       gpointer              user_data);
-static void     initialize_removable_devices  (TrackerMinerFiles    *mf);
 
-static void     on_battery_cb                 (GObject              *gobject,
-					       GParamSpec           *arg1,
-					       gpointer              user_data);
-static void     on_low_battery_cb             (GObject              *object,
+static void     check_battery_status          (TrackerMinerFiles    *fs);
+static void     battery_status_cb             (GObject              *object,
 					       GParamSpec           *pspec,
 					       gpointer              user_data);
-static void     on_battery_percentage_cb      (GObject              *gobject,
-					       GParamSpec           *arg1,
-					       gpointer              user_data);
 #endif
 
 static void     init_mount_points             (TrackerMinerFiles *miner);
@@ -141,9 +133,7 @@ static gboolean miner_files_check_directory_contents (TrackerMinerFS       *fs,
 static gboolean miner_files_process_file      (TrackerMinerFS       *fs,
 					       GFile                *file,
 					       TrackerSparqlBuilder *sparql,
-					       GCancellable         *cancellable,
-					       TrackerMinerFSDoneCb  done_cb,
-					       gpointer              done_cb_data);
+					       GCancellable         *cancellable);
 static gboolean miner_files_monitor_directory (TrackerMinerFS       *fs,
 					       GFile                *file);
 
@@ -200,13 +190,10 @@ tracker_miner_files_init (TrackerMinerFiles *mf)
 	priv->power = tracker_power_new ();
 
 	g_signal_connect (priv->power, "notify::on-low-battery",
-			  G_CALLBACK (on_low_battery_cb),
+			  G_CALLBACK (battery_status_cb),
 			  mf);
 	g_signal_connect (priv->power, "notify::on-battery",
-			  G_CALLBACK (on_battery_cb),
-			  mf);
-	g_signal_connect (priv->power, "notify::battery-percentage",
-			  G_CALLBACK (on_battery_percentage_cb),
+			  G_CALLBACK (battery_status_cb),
 			  mf);
 #endif /* HAVE_HAL */
 
@@ -300,21 +287,7 @@ miner_files_finalize (GObject *object)
 	disk_space_check_stop (TRACKER_MINER_FILES (object));
 
 #ifdef HAVE_HAL
-	g_signal_handlers_disconnect_by_func (priv->power,
-					      on_battery_percentage_cb,
-					      mf);
-	g_signal_handlers_disconnect_by_func (priv->power,
-					      on_battery_cb,
-					      mf);
-	g_signal_handlers_disconnect_by_func (priv->power,
-					      on_low_battery_cb,
-					      mf);
         g_object_unref (priv->power);
-
-	g_signal_handlers_disconnect_by_func (priv->storage,
-					      mount_point_added_cb,
-					      mf);
-
         g_object_unref (priv->storage);
 #endif /* HAVE_HAL */
 
@@ -332,7 +305,7 @@ miner_files_constructed (GObject *object)
         TrackerMinerFiles *mf;
         TrackerMinerFS *fs;
         GSList *dirs;
-	gint throttle;
+	GSList *mounts = NULL, *m;
 
 	G_OBJECT_CLASS (tracker_miner_files_parent_class)->constructed (object);
 
@@ -344,10 +317,20 @@ miner_files_constructed (GObject *object)
                 g_assert_not_reached ();
         }
 
+#ifdef HAVE_HAL
+        if (tracker_config_get_index_removable_devices (mf->private->config)) {
+                mounts = tracker_storage_get_removable_device_roots (mf->private->storage);
+        }
+
+	check_battery_status (mf);
+#endif /* HAVE_HAL */
+
+	g_message ("Setting up directories to iterate from config (IndexSingleDirectory)");
+
         /* Fill in directories to inspect */
         dirs = tracker_config_get_index_single_directories (mf->private->config);
 
-        while (dirs) {
+        for (; dirs; dirs = dirs->next) {
 		GFile *file;
 
 		/* Do some simple checks for silly locations */
@@ -362,16 +345,32 @@ miner_files_constructed (GObject *object)
 			continue;
 		}
 
+		/* Make sure we don't crawl volumes. */
+		if (mounts) {
+			gboolean found = FALSE;
+
+			for (m = mounts; m && !found; m = m->next) {
+				found = strcmp (m->data, dirs->data) == 0;
+			}
+
+			if (found) {
+				g_message ("  Duplicate found:'%s' - same as removable device path",
+					   (gchar*) dirs->data);
+				continue;
+			}
+		}
+
+		g_message ("  Adding:'%s'", (gchar*) dirs->data);
 		file = g_file_new_for_path (dirs->data);
                 tracker_miner_fs_add_directory (fs, file, FALSE);
 		g_object_unref (file);
-
-                dirs = dirs->next;
         }
 
+	g_message ("Setting up directories to iterate from config (IndexRecursiveDirectory)");
+
         dirs = tracker_config_get_index_recursive_directories (mf->private->config);
 
-        while (dirs) {
+        for (; dirs; dirs = dirs->next) {
 		GFile *file;
 
 		/* Do some simple checks for silly locations */
@@ -386,29 +385,65 @@ miner_files_constructed (GObject *object)
 			continue;
 		}
 
+		/* Make sure we don't crawl volumes. */
+		if (mounts) {
+			gboolean found = FALSE;
+
+			for (m = mounts; m && !found; m = m->next) {
+				found = strcmp (m->data, dirs->data) == 0;
+			}
+
+			if (found) {
+				g_message ("  Duplicate found:'%s' - same as removable device path",
+					   (gchar*) dirs->data);
+				continue;
+			}
+		}
+
+		g_message ("  Adding:'%s'", (gchar*) dirs->data);
 		file = g_file_new_for_path (dirs->data);
                 tracker_miner_fs_add_directory (fs, file, TRUE);
 		g_object_unref (file);
-
-                dirs = dirs->next;
         }
 
-	throttle = tracker_config_get_throttle (mf->private->config);
+	/* Add removable media */
+	g_message ("Setting up directories to iterate which are removable devices");
 
-	/* Throttle in config goes from 0 to 20, translate to 0.0 -> 1.0 */
-	tracker_miner_fs_set_throttle (TRACKER_MINER_FS (mf), (1.0 / 20) * throttle);
-
-#ifdef HAVE_HAL
-        initialize_removable_devices (mf);
-#endif /* HAVE_HAL */
+	for (m = mounts; m; m = m->next) {
+		GFile *file = g_file_new_for_path (m->data);
+		
+		g_message ("  Adding:'%s'", (gchar*) m->data);
+		tracker_miner_fs_add_directory (TRACKER_MINER_FS (mf),
+						file, 
+						TRUE);
+		g_object_unref (file);
+	}
 
 	g_signal_connect (mf->private->config, "notify::low-disk-space-limit",
 			  G_CALLBACK (low_disk_space_limit_cb),
 			  mf);
 
+	g_slist_foreach (mounts, (GFunc) g_free, NULL);
+	g_slist_free (mounts);
+
 	disk_space_check_start (mf);
 }
 
+static void
+set_up_mount_point_cb (TrackerMiner *miner,
+		       const GError *error,
+		       gpointer      user_data)
+{
+	gchar *removable_device_urn = user_data;
+
+	if (error) {
+		g_critical ("Could not set up mount point '%s': %s",
+			    removable_device_urn, error->message);
+	}
+
+	g_free (removable_device_urn);
+}
+
 static void
 set_up_mount_point (TrackerMinerFiles *miner,
                     const gchar       *removable_device_urn,
@@ -417,7 +452,6 @@ set_up_mount_point (TrackerMinerFiles *miner,
                     GString           *accumulator)
 {
 	GString *queries;
-	GError *error = NULL;
 
 	g_debug ("Setting up mount point '%s'", removable_device_urn);
 
@@ -480,58 +514,59 @@ set_up_mount_point (TrackerMinerFiles *miner,
 	if (accumulator) {
 		g_string_append_printf (accumulator, "%s ", queries->str);
 	} else {
-
-		tracker_miner_execute_update (TRACKER_MINER (miner), queries->str, &error);
-
-		if (error) {
-			g_critical ("Could not set up mount point '%s': %s",
-			            removable_device_urn, error->message);
-			g_error_free (error);
-		}
+		tracker_miner_execute_update (TRACKER_MINER (miner),
+					      queries->str,
+					      NULL,
+					      set_up_mount_point_cb,
+					      g_strdup (removable_device_urn));
 	}
 
 	g_string_free (queries, TRUE);
 }
 
 static void
-init_mount_points (TrackerMinerFiles *miner)
+init_mount_points_cb (TrackerMiner *miner,
+		      const GError *error,
+		      gpointer      user_data)
+{
+	if (error) {
+		g_critical ("Could not initialize currently active mount points: %s",
+			    error->message);
+	}
+}
+
+static void
+query_mount_points_cb (TrackerMiner *miner,
+		       GPtrArray    *result,
+		       const GError *error,
+		       gpointer      user_data)
 {
 	TrackerMinerFilesPrivate *priv;
 	GHashTable *volumes;
 	GHashTableIter iter;
 	gpointer key, value;
-	GPtrArray *sparql_result;
-	GError *error = NULL;
 	GString *accumulator;
 	gint i;
 #ifdef HAVE_HAL
-	GList *udis, *u;
+	GSList *udis, *u;
 #endif
 
-	priv = TRACKER_MINER_FILES_GET_PRIVATE (miner);
+	if (error) {
+		g_critical ("Could not obtain the mounted volumes: %s", error->message);
+		return;
+	}
 
-	g_debug ("Initializing mount points");
+	priv = TRACKER_MINER_FILES_GET_PRIVATE (miner);
 
 	volumes = g_hash_table_new_full (g_str_hash, g_str_equal,
 	                                 (GDestroyNotify) g_free,
 	                                 NULL);
 
-	/* First, get all mounted volumes, according to tracker-store */
-	sparql_result = tracker_miner_execute_sparql (TRACKER_MINER (miner),
-	                                              "SELECT ?v WHERE { ?v a tracker:Volume ; tracker:isMounted true }",
-	                                              &error);
-
-	if (error) {
-		g_critical ("Could not obtain the mounted volumes");
-		g_error_free (error);
-		return;
-	}
-
-	for (i = 0; i < sparql_result->len; i++) {
+	for (i = 0; i < result->len; i++) {
 		gchar **row;
 		gint state;
 
-		row = g_ptr_array_index (sparql_result, i);
+		row = g_ptr_array_index (result, i);
 		state = VOLUME_MOUNTED_IN_STORE;
 
 		if (strcmp (row[0], TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN) == 0) {
@@ -542,9 +577,6 @@ init_mount_points (TrackerMinerFiles *miner)
 		g_hash_table_insert (volumes, g_strdup (row[0]), GINT_TO_POINTER (state));
 	}
 
-	g_ptr_array_foreach (sparql_result, (GFunc) g_strfreev, NULL);
-	g_ptr_array_free (sparql_result, TRUE);
-
 	g_hash_table_replace (volumes, g_strdup (TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN),
 	                      GINT_TO_POINTER (VOLUME_MOUNTED));
 
@@ -566,7 +598,8 @@ init_mount_points (TrackerMinerFiles *miner)
 		g_hash_table_replace (volumes, removable_device_urn, GINT_TO_POINTER (state));
 	}
 
-	g_list_free (udis);
+	g_slist_foreach (udis, (GFunc) g_free, NULL);
+	g_slist_free (udis);
 #endif
 
 	accumulator = g_string_new (NULL);
@@ -592,29 +625,41 @@ init_mount_points (TrackerMinerFiles *miner)
 
 			g_debug ("URN '%s' (mount point: %s) was not reported to be mounted, but now it is, updating state",
 			         mount_point, urn);
-			set_up_mount_point (miner, urn, mount_point, TRUE, accumulator);
+			set_up_mount_point (TRACKER_MINER_FILES (miner), urn, mount_point, TRUE, accumulator);
 		} else if (!(state & VOLUME_MOUNTED) &&
 			   (state & VOLUME_MOUNTED_IN_STORE)) {
 			g_debug ("URN '%s' was reported to be mounted, but it isn't anymore, updating state", urn);
-			set_up_mount_point (miner, urn, NULL, FALSE, accumulator);
+			set_up_mount_point (TRACKER_MINER_FILES (miner), urn, NULL, FALSE, accumulator);
 		}
 	}
 
 	if (accumulator->str[0] != '\0') {
-		tracker_miner_execute_update (TRACKER_MINER (miner), accumulator->str, &error);
-
-		if (error) {
-			g_critical ("Could not initialize currently active mount points: %s",
-			            error->message);
-			g_error_free (error);
-		}
+		tracker_miner_execute_update (miner,
+					      accumulator->str,
+					      NULL,
+					      init_mount_points_cb,
+					      NULL);
 	}
 
 	g_string_free (accumulator, TRUE);
 	g_hash_table_unref (volumes);
 }
 
+static void
+init_mount_points (TrackerMinerFiles *miner)
+{
+	g_debug ("Initializing mount points");
+
+	/* First, get all mounted volumes, according to tracker-store */
+	tracker_miner_execute_sparql (TRACKER_MINER (miner),
+				      "SELECT ?v WHERE { ?v a tracker:Volume ; tracker:isMounted true }",
+				      NULL,
+				      query_mount_points_cb,
+				      NULL);
+}
+
 #ifdef HAVE_HAL
+
 static void
 mount_point_removed_cb (TrackerStorage *storage,
 			const gchar    *udi,
@@ -665,28 +710,6 @@ mount_point_added_cb (TrackerStorage *storage,
 	g_free (urn);
 }
 
-static void
-initialize_removable_devices (TrackerMinerFiles *mf)
-{
-        TrackerMinerFilesPrivate *priv;
-
-        priv = TRACKER_MINER_FILES_GET_PRIVATE (mf);
-
-        if (tracker_config_get_index_removable_devices (priv->config)) {
-                GList *mounts, *m;
-
-                mounts = tracker_storage_get_removable_device_roots (priv->storage);
-
-                for (m = mounts; m; m = m->next) {
-			GFile *as_file = g_file_new_for_path (m->data);
-			tracker_miner_fs_add_directory (TRACKER_MINER_FS (mf),
-							as_file, 
-							TRUE);
-			g_object_unref (as_file);
-                }
-        }
-}
-
 static void
 set_up_throttle (TrackerMinerFiles *mf,
 		 gboolean           enable)
@@ -708,59 +731,35 @@ set_up_throttle (TrackerMinerFiles *mf,
 }
 
 static void
-on_battery_cb (GObject    *gobject,
-	       GParamSpec *arg1,
-	       gpointer    user_data)
+check_battery_status (TrackerMinerFiles *mf)
 {
-	TrackerMinerFiles *mf = user_data;
-	gboolean on_battery;
-
-	set_up_throttle (mf, TRUE);
-	on_battery = tracker_power_get_on_battery (mf->private->power);
-
-	if (on_battery) {
-		/* We only pause on low battery */
-		return;
-	}
-
-	/* Resume if we are not on battery */
-	if (mf->private->low_battery_pause_cookie != 0) {
-		tracker_miner_resume (TRACKER_MINER (mf),
-				      mf->private->low_battery_pause_cookie,
-				      NULL);
-		mf->private->low_battery_pause_cookie = 0;
-	}
-}
-
-static void
-on_low_battery_cb (GObject    *object,
-		   GParamSpec *pspec,
-		   gpointer    user_data)
-{
-	TrackerMinerFiles *mf = user_data;
-	gboolean on_low_battery;
-	gboolean on_battery;
+	gboolean on_battery, on_low_battery;
 	gboolean should_pause = FALSE;
+	gboolean should_throttle = FALSE;
 
 	on_low_battery = tracker_power_get_on_low_battery (mf->private->power);
 	on_battery = tracker_power_get_on_battery (mf->private->power);
 
-	if (on_battery && on_low_battery) {
-		gdouble percentage;
+	if (!on_battery) {
+		g_message ("Running on AC power");
+		should_pause = FALSE;
+		should_throttle = FALSE;
+	} else {
+		g_message ("Running on battery");
 
-		should_pause = TRUE;
-		percentage = tracker_power_get_battery_percentage (mf->private->power);
+		should_throttle = TRUE;
 
-		g_message ("WARNING: Available battery power is getting low (%.0f%%)",
-			   percentage * 100);
+		if (on_low_battery) {
+			g_message ("  Battery is LOW, pausing");
+			should_pause = TRUE;
+		}
 	}
 
 	if (should_pause) {
 		/* Don't try to pause again */
 		if (mf->private->low_battery_pause_cookie == 0) {
-			mf->private->low_battery_pause_cookie = 
+			mf->private->low_battery_pause_cookie =
 				tracker_miner_pause (TRACKER_MINER (mf),
-						     g_get_application_name (),
 						     _("Low battery"),
 						     NULL);
 		}
@@ -774,21 +773,17 @@ on_low_battery_cb (GObject    *object,
 		}
 	}
 
-	set_up_throttle (mf, FALSE);
+	set_up_throttle (mf, should_throttle);
 }
 
 static void
-on_battery_percentage_cb (GObject    *gobject,
-			  GParamSpec *arg1,
-			  gpointer    user_data)
+battery_status_cb (GObject    *object,
+		   GParamSpec *pspec,
+		   gpointer    user_data)
 {
 	TrackerMinerFiles *mf = user_data;
-	gdouble percentage;
 
-	percentage = tracker_power_get_battery_percentage (mf->private->power);
-
-	g_message ("Battery percentage is now %.0f%%",
-		   percentage * 100);
+	check_battery_status (mf);
 }
 
 #endif /* HAVE_HAL */
@@ -848,7 +843,6 @@ disk_space_check_cb (gpointer user_data)
 		if (mf->private->disk_space_pause_cookie == 0) {
 			mf->private->disk_space_pause_cookie = 
 				tracker_miner_pause (TRACKER_MINER (mf),
-						     g_get_application_name (),
 						     _("Low disk space"),
 						     NULL);
 		}
@@ -1208,7 +1202,7 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
 
 	if (error) {
 		/* Something bad happened, notify about the error */
-		data->callback (TRACKER_MINER_FS (data->miner), data->file, data->sparql, error, data->callback_data);
+		tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), data->file, error);
 		process_file_data_free (data);
 		g_error_free (error);
 		return;
@@ -1220,9 +1214,8 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
 		g_free (sparql);
 	}
 
-
 	/* Notify about the success */
-	data->callback (TRACKER_MINER_FS (data->miner), data->file, data->sparql, NULL, data->callback_data);
+	tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), data->file, NULL);
 	process_file_data_free (data);
 }
 
@@ -1237,7 +1230,7 @@ extractor_get_embedded_metadata_cancel (GCancellable    *cancellable,
 				  data->call);
 
 	error = g_error_new_literal (miner_files_error_quark, 0, "Embedded metadata extraction was cancelled");
-	data->callback (TRACKER_MINER_FS (data->miner), data->file, data->sparql, error, data->callback_data);
+	tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), data->file, error);
 
 	process_file_data_free (data);
 	g_error_free (error);
@@ -1278,7 +1271,7 @@ process_file_cb (GObject      *object,
 
 	if (error) {
 		/* Something bad happened, notify about the error */
-		data->callback (TRACKER_MINER_FS (data->miner), file, sparql, error, data->callback_data);
+		tracker_miner_fs_notify_file (TRACKER_MINER_FS (data->miner), file, error);
 		process_file_data_free (data);
 		return;
 	}
@@ -1347,16 +1340,12 @@ static gboolean
 miner_files_process_file (TrackerMinerFS       *fs,
 			  GFile                *file,
 			  TrackerSparqlBuilder *sparql,
-			  GCancellable         *cancellable,
-			  TrackerMinerFSDoneCb  done_cb,
-			  gpointer              done_cb_data)
+			  GCancellable         *cancellable)
 {
 	ProcessFileData *data;
 	const gchar *attrs;
 
 	data = g_slice_new0 (ProcessFileData);
-	data->callback = done_cb;
-	data->callback_data = done_cb_data;
 	data->miner = g_object_ref (fs);
 	data->cancellable = g_object_ref (cancellable);
 	data->sparql = g_object_ref (sparql);
@@ -1386,5 +1375,6 @@ tracker_miner_files_new (TrackerConfig *config)
         return g_object_new (TRACKER_TYPE_MINER_FILES,
                              "name", "Files",
                              "config", config,
+			     "process-pool-limit", 10,
                              NULL);
 }
