@@ -53,7 +53,6 @@
 #include "tracker-dbus.h"
 #include "tracker-config.h"
 #include "tracker-events.h"
-#include "tracker-main.h"
 #include "tracker-push.h"
 #include "tracker-backup.h"
 #include "tracker-store.h"
@@ -76,31 +75,27 @@
 	"  http://www.gnu.org/licenses/gpl.txt\n"
 
 typedef struct {
-	GMainLoop	 *main_loop;
-	gchar		 *log_filename;
+	GMainLoop *main_loop;
+	gchar *log_filename;
 
-	gchar		 *data_dir;
-	gchar		 *user_data_dir;
-	gchar		 *sys_tmp_dir;
-	gchar            *ttl_backup_file;
+	gchar *ttl_backup_file;
 
-	gboolean          first_time_index;
-	gboolean	  reindex_on_shutdown;
-	gboolean          shutdown;
+	gboolean first_time_index;
+	gboolean reindex_on_shutdown;
+	gboolean shutdown;
 } TrackerMainPrivate;
 
 /* Private */
-static GStaticPrivate	     private_key = G_STATIC_PRIVATE_INIT;
+static GStaticPrivate private_key = G_STATIC_PRIVATE_INIT;
 
 /* Private command line parameters */
-static gboolean		     version;
-static gint		     verbosity = -1;
-static gboolean		     low_memory;
+static gboolean version;
+static gint verbosity = -1;
+static gboolean low_memory;
+static gboolean force_reindex;
+static gboolean readonly_mode;
 
-static gboolean		     force_reindex;
-static gboolean		     readonly_mode;
-
-static GOptionEntry	     entries[] = {
+static GOptionEntry  entries[] = {
 	/* Daemon options */
 	{ "version", 'V', 0,
 	  G_OPTION_ARG_NONE, &version,
@@ -134,10 +129,6 @@ private_free (gpointer data)
 
 	private = data;
 
-	g_free (private->sys_tmp_dir);
-	g_free (private->user_data_dir);
-	g_free (private->data_dir);
-
 	g_free (private->ttl_backup_file);
 	g_free (private->log_filename);
 
@@ -160,6 +151,22 @@ sanity_check_option_values (TrackerConfig *config)
 		   readonly_mode ? "yes" : "no");
 }
 
+static void
+shutdown (void)
+{
+	TrackerMainPrivate *private;
+
+	private = g_static_private_get (&private_key);
+
+	if (private) {
+		if (private->main_loop) {
+			g_main_loop_quit (private->main_loop);
+		}
+
+		private->shutdown = TRUE;
+	}
+}
+
 static gboolean
 shutdown_timeout_cb (gpointer user_data)
 {
@@ -183,7 +190,7 @@ signal_handler (int signo)
 	case SIGTERM:
 	case SIGINT:
 		in_loop = TRUE;
-		tracker_shutdown ();
+		shutdown ();
 
 	default:
 		if (g_strsignal (signo)) {
@@ -226,70 +233,12 @@ initialize_priority (void)
 	 */
 }
 
-static void
-initialize_locations (void)
-{
-	TrackerMainPrivate *private;
-	gchar		   *filename;
-
-	private = g_static_private_get (&private_key);
-
-	/* Public locations */
-	private->user_data_dir =
-		g_build_filename (g_get_user_data_dir (),
-				  "tracker",
-				  "data",
-				  NULL);
-
-	private->data_dir =
-		g_build_filename (g_get_user_cache_dir (),
-				  "tracker",
-				  NULL);
-
-	filename = g_strdup_printf ("tracker-%s", g_get_user_name ());
-	private->sys_tmp_dir =
-		g_build_filename (g_get_tmp_dir (),
-				  filename,
-				  NULL);
-	g_free (filename);
-
-	private->ttl_backup_file = 
-		g_build_filename (private->user_data_dir, 
-				  "tracker-userdata-backup.ttl",
-				  NULL);
-}
-
 static void
 initialize_directories (void)
 {
-	TrackerMainPrivate *private;
-	gchar		   *filename;
-
-	private = g_static_private_get (&private_key);
-
 	/* NOTE: We don't create the database directories here, the
 	 * tracker-db-manager does that for us.
 	 */
-
-	g_message ("Checking directory exists:'%s'", private->user_data_dir);
-	g_mkdir_with_parents (private->user_data_dir, 00755);
-
-	g_message ("Checking directory exists:'%s'", private->data_dir);
-	g_mkdir_with_parents (private->data_dir, 00755);
-
-	/* Remove old tracker dirs */
-	filename = g_build_filename (g_get_home_dir (), ".Tracker", NULL);
-
-	if (g_file_test (filename, G_FILE_TEST_EXISTS)) {
-		tracker_path_remove (filename);
-	}
-
-	g_free (filename);
-
-	/* Remove database if we are reindexing */
-	filename = g_build_filename (private->sys_tmp_dir, "Attachments", NULL);
-	g_mkdir_with_parents (filename, 00700);
-	g_free (filename);
 }
 
 static void
@@ -308,12 +257,6 @@ shutdown_databases (void)
 #endif
 }
 
-static void
-shutdown_locations (void)
-{
-	/* Nothing to do here, this is done by the private free func */
-}
-
 static void
 shutdown_directories (void)
 {
@@ -327,54 +270,11 @@ shutdown_directories (void)
 	}
 }
 
-#if 0
-
-static const gchar *
-get_ttl_backup_filename (void) 
-{
-	TrackerMainPrivate *private;
-
-	private = g_static_private_get (&private_key);
-
-	return private->ttl_backup_file;
-}
-
-static void
-backup_user_metadata (TrackerConfig   *config, 
-		      TrackerLanguage *language)
-{
-	gboolean is_first_time_index;
-
-	g_message ("Saving metadata in %s", get_ttl_backup_filename ());
-	
-	/*
-	 *  Init the DB stack to get the user metadata
-	 */
-
-	tracker_data_manager_init (0, NULL, &is_first_time_index);
-	
-	/*
-	 * If some database is missing or the dbs dont exists, we dont need
-	 * to backup anything.
-	 */
-	if (is_first_time_index) {
-		tracker_data_manager_shutdown ();
-		return;
-	}
-
-	/* Actual save of the metadata */
-	tracker_data_backup_save (get_ttl_backup_filename (), NULL);
-	
-	/* Shutdown the DB stack */
-	tracker_data_manager_shutdown ();
-}
-#endif
-
 static GStrv
-tracker_daemon_get_notifiable_classes (void)
+get_notifiable_classes (void)
 {
 	TrackerDBResultSet *result_set;
-	GStrv               classes_to_signal = NULL;
+	GStrv classes_to_signal = NULL;
 
 	result_set = tracker_data_query_sparql ("SELECT ?class WHERE { ?class tracker:notify true }", NULL);
 
@@ -391,12 +291,13 @@ tracker_daemon_get_notifiable_classes (void)
 gint
 main (gint argc, gchar *argv[])
 {
-	GOptionContext		   *context = NULL;
-	GError			   *error = NULL;
-	TrackerMainPrivate	   *private;
-	TrackerConfig		   *config;
-	TrackerDBManagerFlags	    flags = 0;
-	gboolean		    is_first_time_index, need_journal = FALSE;
+	GOptionContext *context = NULL;
+	GError *error = NULL;
+	TrackerMainPrivate *private;
+	TrackerConfig *config;
+	TrackerDBManagerFlags flags = 0;
+	gboolean is_first_time_index;
+	gboolean need_journal;
 
 	g_type_init ();
 
@@ -452,14 +353,13 @@ main (gint argc, gchar *argv[])
 	/* This makes sure we don't steal all the system's resources */
 	initialize_priority ();
 
-	/* This makes sure we have all the locations like the data
-	 * dir, user data dir, etc all configured.
-	 *
-	 * The initialize_directories() function makes sure everything
-	 * exists physically and/or is reset depending on various
-	 * options (like if we reindex, we remove the data dir).
-	 */
-	initialize_locations ();
+	/* Public locations */
+	private->ttl_backup_file = 
+		g_build_filename (g_get_user_data_dir (),
+				  "tracker",
+				  "data",
+				  "tracker-userdata-backup.ttl",
+				  NULL);
 
 	/* Initialize major subsystems */
 	config = tracker_config_new ();
@@ -499,7 +399,9 @@ main (gint argc, gchar *argv[])
 		flags |= TRACKER_DB_MANAGER_LOW_MEMORY_MODE;
 	}
 
-	if (!tracker_data_manager_init (flags, NULL, &is_first_time_index, 
+	if (!tracker_data_manager_init (flags, 
+					NULL, 
+					&is_first_time_index, 
 	                                &need_journal)) {
 		return EXIT_FAILURE;
 	}
@@ -515,7 +417,7 @@ main (gint argc, gchar *argv[])
 		return EXIT_FAILURE;
 	}
 
-	tracker_events_init (tracker_daemon_get_notifiable_classes);
+	tracker_events_init (get_notifiable_classes);
 	tracker_push_init ();
 
 	g_message ("Waiting for D-Bus requests...");
@@ -538,14 +440,12 @@ shutdown:
 
 	g_timeout_add_full (G_PRIORITY_LOW, 5000, shutdown_timeout_cb, NULL, NULL);
 
-
 	g_message ("Cleaning up");
 
 	shutdown_databases ();
 	shutdown_directories ();
 
 	/* Shutdown major subsystems */
-
 	tracker_push_shutdown ();
 	tracker_events_shutdown ();
 
@@ -555,56 +455,7 @@ shutdown:
 
 	g_object_unref (config);
 
-	shutdown_locations ();
-
 	g_print ("\nOK\n\n");
 
 	return EXIT_SUCCESS;
 }
-
-void
-tracker_shutdown (void)
-{
-	TrackerMainPrivate *private;
-
-	private = g_static_private_get (&private_key);
-
-	if (private) {
-		if (private->main_loop) {
-			g_main_loop_quit (private->main_loop);
-		}
-
-		private->shutdown = TRUE;
-	}
-}
-
-const gchar *
-tracker_get_data_dir (void)
-{
-	TrackerMainPrivate *private;
-
-	private = g_static_private_get (&private_key);
-
-	return private->data_dir;
-}
-
-const gchar *
-tracker_get_sys_tmp_dir (void)
-{
-	TrackerMainPrivate *private;
-
-	private = g_static_private_get (&private_key);
-
-	return private->sys_tmp_dir;
-}
-
-void
-tracker_set_reindex_on_shutdown (gboolean value)
-{
-	TrackerMainPrivate *private;
-
-	private = g_static_private_get (&private_key);
-
-	private->reindex_on_shutdown = value;
-}
-
